﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis.Symbolic {
  /// <summary>
  /// An operator which clones and assigns the value of one parameter to another parameter.
  /// </summary>
  [Item("ScopeTreeAssigner", "An operator which clones and assigns the value of one parameter to another parameter.")]
  [StorableClass]
  public sealed class ScopeTreeAssigner<T> : SingleSuccessorOperator where T : class, IItem {
    public ScopeTreeLookupParameter<T> LeftSideParameter {
      get { return (ScopeTreeLookupParameter<T>)Parameters["LeftSide"]; }
    }
    public IValueLookupParameter<ItemArray<T>> RightSideParameter {
      get { return (IValueLookupParameter<ItemArray<T>>)Parameters["RightSide"]; }
    }

    [StorableConstructor]
    private ScopeTreeAssigner(bool deserializing) : base(deserializing) { }
    private ScopeTreeAssigner(ScopeTreeAssigner<T> original, Cloner cloner) : base(original, cloner) { }

    public ScopeTreeAssigner()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<T>("LeftSide", "The parameter whose value gets assigned with a clone of the other parameter's value."));
      Parameters.Add(new ValueLookupParameter<ItemArray<T>>("RightSide", "The parameter whose value is cloned and assigned to the value of the other parameter."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ScopeTreeAssigner<T>(this, cloner);
    }

    public override IOperation Apply() {
      LeftSideParameter.ActualValue = (ItemArray<T>)RightSideParameter.ActualValue.Clone();
      return base.Apply();
    }
  }
}
